import java.util.*;
import java.io.*;

public class sched {
	static LinkedList runq = new LinkedList();

	public static void main(String[] args) {
		InputStreamReader rd = new InputStreamReader(System.in);
		BufferedReader stdin = new BufferedReader(rd);

		read_procs(stdin);
		if(runq.isEmpty()) {
			System.err.println("no processes entered, aborting");
			System.exit(1);
		}

		System.out.println("\n--- shortest job first ---");
		shortest_job_first();

		System.out.println("\n--- round-robin ---");
		round_robin(1);		// 1 is the timeslice, maybe we should get this as a cmdline arg

		System.exit(0);
	}

	static void read_procs(BufferedReader in) {
		System.out.println("enter the processes, one line each, followed by an empty line or EOF to end");
		System.out.println("format: length start_time");

		try {
			int id = 0;
			String line;
			while((line = in.readLine()) != null && line.length() > 0) {
				Process proc = new Process(id);
				if(proc.read_desc(line)) {
					runq.add(proc);
					id++;
				}
			}
		}
		catch(IOException e) {}	// that's ok, just got an EOF, so stop reading.
	}


	static void shortest_job_first() {
		LinkedList runq = (LinkedList)sched.runq.clone();	// so we can cannibalize it without affecting the original
		int pcount = runq.size();

		int[] response = new int[pcount];
		int[] turn_around = new int[pcount];

		int resp_sum = 0;
		int turn_sum = 0;
		int cur_time = 0;

		while(!runq.isEmpty()) {	// run until all processes are done
			Process run_proc = null;
			int run_idx = 0;

			ListIterator iter = runq.listIterator();
			while(iter.hasNext()) {
				int idx = iter.nextIndex();
				Process proc = (Process)iter.next();

				if(proc.start() <= cur_time && (run_proc == null || proc.length() < run_proc.length())) {
					run_proc = proc;
					run_idx = idx;
				}
			}

			if(run_proc != null) {	// found a process to run
				int pid = run_proc.pid();

				response[pid] = cur_time;
				resp_sum += cur_time;

				System.out.println("[" + cur_time + "] run(" + pid + ")");

				cur_time += run_proc.length();
				turn_around[pid] = cur_time;
				turn_sum += cur_time;

				runq.remove(run_idx);
			}
		}

		float avg_resp = (float)resp_sum / (float)pcount;
		float avg_turn = (float)turn_sum / (float)pcount;

		for(int i=0; i<pcount; i++) {
			System.out.println("pid: " + i + "  response: " + response[i] + "  turn around: " + turn_around[i]);
		}

		System.out.println("\nAverage response: " + avg_resp);
		System.out.println("Average turnaround: " + avg_turn + "\n");
	}

	static void round_robin(int timeslice) {
		LinkedList runq = (LinkedList)sched.runq.clone();	// so we can cannibalize it without affecting the original
		int pcount = runq.size();
		
		int[] response = new int[pcount];
		int[] turn_around = new int[pcount];
		int resp_sum = 0;
		int turn_sum = 0;

		// keeps the run-time for each process
		int[] runtime = new int[pcount];
		for(int i=0; i<pcount; i++) {
			runtime[i] = 0;
		}

		// run the simulation
		int cur_time = 0;
		while(!runq.isEmpty()) {
			Process run_proc = null;

			ListIterator iter = runq.listIterator();
			while(iter.hasNext()) {
				int idx = iter.nextIndex();
				Process proc = (Process)iter.next();

				if(proc.start() <= cur_time) {
					run_proc = proc;
					runq.remove(proc);	// remove it from the queue temproarilly
					break;
				}
			}

			if(run_proc != null) {	// found a process to run
				int pid = run_proc.pid();

				if(runtime[pid] == 0) {
					response[pid] = cur_time;
					resp_sum += cur_time;
				}

				System.out.println("[" + cur_time + "] run(" + pid + ")");

				cur_time += timeslice;
				runtime[pid] += timeslice;

				if(runtime[pid] < run_proc.length()) {	// didn't finish, reinsert at the back
					runq.add(run_proc);
				} else {
					// finished, don't reinsert, keep turnaround stats
					turn_around[pid] = cur_time;
					turn_sum += cur_time;
				}
			}

		}

		float avg_resp = (float)resp_sum / (float)pcount;
		float avg_turn = (float)turn_sum / (float)pcount;

		for(int i=0; i<pcount; i++) {
			System.out.println("pid: " + i + "  response: " + response[i] + "  turn around: " + turn_around[i]);
		}

		System.out.println("\nAverage response: " + avg_resp);
		System.out.println("Average turnaround: " + avg_turn + "\n");
	}
}

class Process {
	int pid;
	int len;
	int start;

	public Process(int pid) {
		this.pid = pid;
		len = start = 0;
	}

	public int pid() { return pid; }
	public int length() { return len; }
	public int start() { return start; }

	boolean read_desc(String pdesc) {
		int idx = 0;
		StringTokenizer strtok = new StringTokenizer(pdesc);
		
		while(strtok.hasMoreTokens()) {
			String tok = strtok.nextToken();
			if(idx >= 2) {
				System.err.println("unexpected substring: " + tok);
				return false;
			}

			try {
				int val = Integer.parseInt(tok);

				if(idx++ == 0) {
					len = val;
				} else {
					start = val;
				}
			}
			catch(NumberFormatException e) {
				System.err.println("expected a number, got: " + tok);
				return false;
			}
		}

		if(idx == 2) {
			return true;
		}
		System.err.println("short description string (" + idx + "): " + pdesc);
		return false;
	}
}
